(function ($, PHD, window, undefined) {
    /*
    *   Handles CRUD and events for widget- or page-based User Preferences
    */

    const defaults = {
        name: 'PHD Virtual Backup',
        namespace: 'phduserpreference',
        key: 0,
        defaultValue: 0,
        modal: false
    };

    const dialogOptions = {
        autoOpen: true,
        modal: false,
        show: { effect: 'slide', direction: 'up' },
        hide: { effect: 'scale', percent: 0, origin: ['top', 'right'] },
        dialogClass: 'user-preference-dialog',
        resizable: false,
        draggable: false,
        width: 262,
        buttons: {}
    };

    const UserPreference = function UserPreference($trigger, options) {
        if (this instanceof UserPreference) {
            options = options || {};
            this.options = $.extend({}, defaults, options);

            this.$trigger = $trigger;
        } else {
            return new UserPreference($trigger, options).init();
        }
    };

    UserPreference.prototype = {
        init() {
            const self = this;

            this.editing = false;

            this.$dialog = $('<div>', {
                id: `dialog-${this.options.namespace}`,
                class: 'dialog-box user-preference'
            });

            this._dialogOptions = $.extend({}, dialogOptions);
            this._dialogOptions.modal = this.options.modal;
            this._dialogOptions.position = {
                my: 'right top',
                at: 'right top',
                of: this.$trigger.parent()
            };
            const elem = $('[ng-controller=GlobalCtrl]');
            this.scope = elem.scope();
            const injector = elem.injector();
            this.gettextCatalog = injector.get('gettextCatalog');

            this.username = this.scope.user.name;
            this._dialogOptions.title = `${this.gettextCatalog.getString(this.options.name)} ${this.gettextCatalog.getString('Preferences')}`;

            this.$trigger.on('click', (event) => {
                if (!self.editing) {
                    self.editing = true;
                    event.preventDefault();
                    event.stopPropagation();
                    self.edit();
                }
            });

            this.$dialog
                .on('click', '.btn-pref-save', $.proxy(this._save, this))
                .on('click', '.btn-pref-cancel', $.proxy(this._cancel, this));

            return this;
        },

        edit() {
            const self = this;

            PHD.Ajax.get(`/api/preferences/${encodeURI(self.username)}/${this.options.key}/`)
                .done((data) => {
                    const preferences = data.data;
                    const rate = preferences[0][self.options.key];
                    PHD.Ajax.getHTML('app/preferences/userpreference.html')
                        .done((html) => {
                            // eslint-disable-next-line
                            angular.element(document).injector().invoke(function ($compile) {
                                html = $compile(html)(self.scope);
                                self.$dialog.html(html);
                                self.$dialog.find('#user-preference-name').text(self.gettextCatalog.getString(self.options.name));
                                self.$dialog.find('#refresh-name').text(self.options.refresh_name);
                                self.$dialog.find('#id_refresh').val(rate);
                                self.$dialog.find('#user-preference-rate').text(rate);
                                self.$dialog.insertAfter(self.$trigger).dialog(self._dialogOptions);
                            });
                        });
                });
        },

        _save() {
            const self = this;

            const $form = this.$dialog.find('form');

            const key = this.options.key;

            const values = $form.serializeArray();

            const value = _.findWhere(values, { name: 'refresh_rates' });

            const putData = {};
            putData[key] = value.value;

            PHD.Ajax.put(`/api/preferences/${encodeURI(self.username)}/`, JSON.stringify(putData))
                .done((data) => {
                    const rate = parseInt(value.value) * 1000;
                    console.log(`trigger namespace ${self.options.namespace} and rate = `, rate);
                    $(document).trigger(`userpreferencesave.${self.options.namespace}`, [key, rate]);
                    self._close();
                })
                .fail((data) => {
                    $(document).trigger(`userpreferencefail.${self.options.namespace}`, [key, value, data]);
                    self._close();
                });
        },

        _cancel() {
            this._close();
        },

        _close() {
            this.editing = false;
            this.$dialog.dialog('destroy').html('').detach();
        },

        cancel() {
            if (this.editing === true) {
                this._cancel();
            }
        }

    };

    PHD.UserPreference = UserPreference;

    const isInt = function (n) {
        return !isNaN(parseInt(n, 10));
    };
}(jQuery, window.PHD || (window.PHD = {}), window));
