(function($) {

    // jQuery functions and extensions
    $.expr[":"].icontains = jQuery.expr.createPseudo(function(arg) {
        return function(Element) {
            return jQuery(Element).text().toUpperCase().indexOf(arg.toUpperCase()) >= 0;
        };
    });

    $.fn.allAre = function(param) {
        var ret = true;
        this.each(function() {
            if(!$(this).is(param)) {
                ret = false;
            }
        });
        return ret;
    };

    $.fn.disableButton = function() {
        return this.addClass("disabled").attr("href", "#").off("click").on("click", function() { return false; });
    };

    $.fn.toggleButton = function(onoff) {
        return this.toggleClass("disabled", !onoff).data("disabled", !onoff);
    };

    $.fn.disable = function() {
        return this.each(function() { this.disabled = true; });
    };

    $.fn.enable = function() {
        return this.each(function() { this.disabled = false; });
    };

    $.fn.toggleDisabled = function() {
        return this.each(function() { this.disabled = !this.disabled; });
    };

    $.fn.hasAttr = function(which) {
        return (this.attr(which) !== void 0 && this.attr(which) !== false);
    };

    // TODO: Decide which one of these readonly methods is superior then mercilessly kill the other
    $.fn.readonly = function(action) {
        return this.each(function() {
            $(this).removeClass("readonly").off("change.readonly");

            if(action !== "disable") {
                var value = this.selectedIndex;
                $(this)
                    .addClass("readonly")
                    .on("change.readonly", function() {
                        this.selectedIndex = value;
                    });
            }
        });
    };

    var $readonlyDummy = $("<input>", {"type": "text", "class": "dummy"}).prop("readonly", true);

    $.fn.readonly = function(action) {
        return this.each(function() {
            if(action !== "disable" && !$(this).data("dummy")) {
                $(this)
                    .addClass("readonly")
                    .data("dummy", $readonlyDummy.clone().val($(this).find(":selected").text()).insertAfter($(this)))
                    .hide()
                    .on("mousedown.readonly", function(event) {
                        event.preventDefault();
                        event.stopPropagation();
                        return false;
                    })
                    .on("mousedown.readonly", "option", function(event) {
                        event.preventDefault();
                        event.stopPropagation();
                        return false;
                    })
                    .on("change.readonly", function() {
                        if(!$(this).val()) {
                            $(this).data("dummy").val("");
                        } else {
                            $(this).data("dummy").val($(this).find(":selected").text());
                        }
                    });
            } else if(action === "disabled") {
                $(this)
                    .removeClass("readonly")
                    .off("mousedown.readonly")
                    .off("change.readonly")
                    .show()
                    .data("dummy").remove();


                $(this).data("dummy", null);
            }
        });
    };

    $.fn.alterClass = function (remove, add) {
        var pattern = new RegExp(
                '\\s' + remove.
                replace( /\*/g, '[A-Za-z0-9-_]+' ).
                split( ' ' ).
                join( '\\s|\\s' ) +
                '\\s', 'g' );

        if (remove.indexOf('*') === -1) {
            this.removeClass(remove);
            return !add ? this : this.addClass(add);
        }

        this.each(function(index, Element) {
            var classname = ' ' + Element.className + ' ';
            while (pattern.test(classname)) {
                classname = classname.replace(pattern, ' ');
            }
            Element.className = $.trim(classname);
        });

        return !add ? this : this.addClass(add);
    };

    $.fn.lockRows = function(widths) {
        widths = (widths === void 0) ? this.data("widths") : widths;
        if(widths === void 0) {
            return this;
        }
        return this.each(function() {
            var $cells = $(this).find("tbody tr:visible").first().children("td"),
                numcols = $cells.length;
            $cells.each(function(index) {
                $(this).css("width", widths[index]);
                if(index != (numcols - 1)) {
                    $(this).css("min-width", widths[index]);
                }
            });
        });
    };

})(jQuery);