(function($) {

    $.fn.sortTable = function(methods) {
        methods = methods || {};

        return this.each(function() {
            var $table = $(this).addClass("sort-table"),
                $tbody = $table.find("tbody");

            methods = $.extend({}, {
                "int": function(a, b) {
                    return parseInt(a.value, 10) - parseInt(b.value, 10);
                },
                "float": function(a, b) {
                    return parseFloat(a.value) - parseFloat(b.value);
                },
                "string": function(a, b) {
                    if(_.isArray(a.value)) {
                        x = a.value.join("").toLowerCase();
                    } else if (!isNaN(a.value)) {
                        x = a.value;
                    } else {
                        x = a.value.toLowerCase();
                    }
                    if(_.isArray(b.value)) {
                        y = b.value.join("").toLowerCase();
                    } else if (!isNaN(b.value)) {
                        y = b.value;
                    } else {
                        y = b.value.toLowerCase();
                    }

                    return (x < y) ?
                        -1 : (x > y) ?
                            1 : 0;
                },
                "date": function(a, b) {
                    if(a.value === EMPTY_CELL_TEXT) {
                        return -1;
                    }
                    if(b.value === EMPTY_CELL_TEXT) {
                        return 1;
                    }
                    x = new Date(a.value);
                    y = new Date(b.value);
                    return (x < y) ?
                        -1 : (x > y) ?
                            1 : 0;
                }
            }, methods);

            // Initialize table headers
            if($table.data("sortTable") !== true) {
                var $defaultSort = $table.find("th[data-sort-dir=asc], th[data-sort-dir=desc]").first(),
                    defaultSortDir = $defaultSort.data("sort-dir");

                $table.data("sort-on", $defaultSort);
                $table.resetSortIndicators();
                $defaultSort.data("sort-dir", defaultSortDir).addClass("sort-" + defaultSortDir);
            }
            
            if($table.data("sortTable") === true) {
                // Enable sorting by calling this method
                var $column = $table.data("sort-on");
                $table.sort($column.index(), methods[$column.data('sort')], $column.data('sort-dir'));
            } else {
                // Enable sorting when a table header is clicked
                $table.on("click.sortTable", "th.sortable", function() {
                    var $header = $(this),
                        methodName = $header.data("sort") || null,
                        direction = $header.data("sort-dir") || null,
                        columnIndex = $header.index(),
                        sortMethod, newSortDir, colValues = [];

                    if(methodName === null)
                        return;

                    sortMethod = methods[methodName];
                    $table.resetSortIndicators();

                    if(direction) {
                        direction = direction === "desc" ? "asc" : "desc";
                        $table.reverseRows();
                    } else {
                        direction = "asc";
                        $table.sort(columnIndex, sortMethod, direction);
                    }

                    $table.data("sort-on", $header);

                    $header
                        .data("sort-dir", direction)
                        .addClass("sort-"+direction);
                    $table.trigger("aftertablesort", [$header, direction]);
                });
            }

            $table.data("sortTable", true);
        });
    };

    $.fn.reverseRows = function($rows) {
        return this.each(function() {
            var $tbody = $(this).children("tbody"),
                $rows = $tbody.find("tr:not(.placeholder)");
            $tbody.append($rows.detach().reverse());
        });
    };

    $.fn.resetSortIndicators = function() {
        return this.find("th.sortable")
                .data("sort-dir", null)
                .removeClass("sort-asc sort-desc")
                .removeAttr("data-sort-dir");
    };

    $.fn.sort = function(columnIndex, method, direction) {
        var sortValues = function(values, method) {
            return values.slice(0).sort(method);
        };

        return this.each(function() {
            var $table = $(this),
                $tbody = $table.children("tbody"),
                $rows = $tbody.find("tr:not(.placeholder)"),
                values = [], sortedValues, $sortedRows = $();

            $rows.each(function(index, row) {
                var $cell = $(row).find("td").eq(columnIndex),
                    sortValue = $cell.data("sort-value");

                if(sortValue === void 0 || sortValue === "undefined") {
                    sortValue = $cell.text();
                }

                values.push({"index": index, "value": sortValue});
            });

            sortedValues = sortValues(values, method);

            if(direction === 'desc') {
                sortedValues.reverse();
            }

            sortedValues.forEach(function(value) {
                $sortedRows = $sortedRows.add($rows.eq(value.index).detach());
            });

            $tbody.append($sortedRows);

            $table.trigger("aftersort", [columnIndex, direction]);
        });
    };

    // Helper Functions
    $.fn.reverse = Array.prototype.reverse;

})(jQuery);