
(function ($, PHD, window, undefined) {

    /*
     *
     *   Defines logic for page-level buttons
     *   @param $but - the button element that was clicked
     *   @param $row - the element or element(s) that are considered "selected" by the user
     *   @param tab - the object that owns $but and $row - typically this is the object that called buttonlogic()
     *
     */

    var jobTypeRegex = /[\s|^]job-type-([a-z]+)(?:\s)?/;

    // Access the gettextCatalog service injected into the app.
    var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
    var injector = elem.injector();
    var gettextCatalog = injector.get('gettextCatalog');


    var jobEditSelections = function(rowData, data) {
        var selections = "";
        console.log("in job edit selections, data = ", data);
        if (rowData != null) {
            var inventory = "inventory=";
            var selection = "";
            var system_id = (rowData.sid != undefined) ? rowData.sid : 1;
            var client_id = 0;
            if (data.client_id != undefined) {
                client_id = data.client_id;
            }
            var app_id = 1;
            if (data.app_id !== undefined) {
                app_id = data.app_id;
            }
            var added = false;
            var items = 0;
            if (data.instances != null) {
                added = true;
                var base_inventory = inventory + system_id + "_" + client_id;
                //console.log("data is " + JSON.stringify(data));
                if (data.instances.length > 0) {
                    console.log("app id is " + app_id);
                    for (var i = 0; i < data.instances.length; i++) {
                        var instance = data.instances[i];
                        if (instance.app_id !== undefined){
                            app_id = instance.app_id;
                        }
                        var to_add = base_inventory + "_" + app_id + "_" + instance.id;
                        if (items > 0) {
                            selection += "&";
                        }
                        items++;
                        selection += to_add;
                    }
                }
            }
            if (data.clients != undefined && data.clients != null && data.clients.length > 0) {
                added = true;
                base_inventory = inventory + system_id;
                for (let i = 0; i < data.clients.length; i++) {
                    client_id = data.clients[i];
                    to_add = base_inventory + "_" + client_id; // + "_" + "1" + instance.id;
                    if (items > 0) {
                        selection += "&";
                    }
                    items++;
                    selection += to_add;
                }
            }
            if (!added) {
                selection = inventory;
            }
            selections += selection;
        }
        console.log("job edit selections: " + selections);
        return selections;
    };

    var getSelectedBackups = function(tab) {
        var selections_obj = {};
        selections_obj.backups = [];
        var backups = [];
        if (tab.TreeTable !== undefined) {
            var selectedBackups = tab.TreeTable.getSelectionsExtra();
            if (selectedBackups.length > 0) {
                //console.log("selected backups = " + JSON.stringify(selectedBackups));
                var backups_per_instance = new Array();
                selectedBackups.forEach(function (obj, index) {
                    if (obj.backup_id !== undefined) {
                        var instance_id = obj.instance_id;
                        if (!$.isArray(backups_per_instance[instance_id])) {
                            backups_per_instance[instance_id] = new Array();
                        }
                        backups_per_instance[instance_id].push(obj.backup_id);
                    }
                });
                for (var instance in backups_per_instance) {
                    var b = backups_per_instance[instance];
                    b.sort();
                    backups.push(b[b.length - 1]);
                }
            }
            selections_obj.backups = backups;
        }
        //console.log("in: getSelectedBackups: selections object = " + JSON.stringify(selections_obj));
        return selections_obj;
    };

    var isFinished = function(status) {
        var finished = false;
        switch (status) {
            case 'Warning':
            case 'Successful':
            case 'Success':
            case 'Error':
            case 'error':
            case 'Complete':
            case 'Process Terminated':
            case 'available':
            case 'Available':
                finished = true;
                break;
        }
        return finished;
    };


    var isCanceled = function(status) {
        var canceled = false;
        switch (status) {
            case 'Unknown/Cancelled':
            case 'Cancelled':
                canceled = true;
                break;
        }
        return canceled;
    };

    var isSuspendable = function(status, type) {
        var suspendable = false;
        switch (status) {
            case 'Queued':
                suspendable = true;
                break;
            case 'Active':
                if (type === 'certification') {
                    suspendable = true;
                }
                break;
        }
        return suspendable;
    };

    var isResumable = function(status, type) {
        var resumable = false;
        switch (status) {
            case 'Paused':
                resumable = true;
                break;
        }
        return resumable;
    };

    PHD.buttonLogic = function ($but, $row, tab, id, row_data, analytics, ids, ngDialog) {
        var rowData = row_data || null;
        var editpath = "", view = "", action = "";
        var $analytics = analytics ? analytics.provider : false;
        var analyticsSource = analytics ? analytics.source : null;
        var selectedIDs = ids || [];
        var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
        var $scope = elem.scope();

        $but.off("click click.autoblur").on("click.autoblur", function () {
            $(this).blur();
        });

        if ($but.is(".btn-edit-job")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot edit jobs");
                return true;
            }
            // No need to check for monitor user as no jobs will be listed in the Job Manager for edit.
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-edit-policy")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot edit SLA policies");
                return true;
            }
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot edit SLA policies");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                $but.attr("href", "app/sla-policies/" + selectedIDs[0])
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-policy")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot delete SLA Policies");
                return true;
            }
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot delete SLA Policies");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                selectedJob = rowData;
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        if (selectedJob.type === 'policy') {
                            var $loadElement = $but;
                            if (_.isElement($row)) {
                                $loadElement = $row.find("[data-source=\"name\"]");
                            }
                            ngDialog.open({
                                template: 'app/protect/sla-policy/delete-policy.html',
                                scope: $scope,
                                overlay:true,
                                data: {'policyId': selectedJob.id, 'selectedPolicy': selectedJob, 'loadSelector': $loadElement, 'analyticsSource': analyticsSource, tab: tab },
                                modelDialogId: 'delete-sla-dialog',
                                name: 'delete-sla-dialog',
                                ngDialogStyle: 'width:625px;height:250px',
                                preCloseCallback: function (value) {
                                    tab.update();
                                }
                            });
                            return true;
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-copy-job")) {
            if ($row !== null && $row.length && !$row.is(".job-type-rollback") && !$row.is(".job-type-replication")) {
                editpath = jobTypeRegex.exec($row.attr("class"))[1];

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.attr("href", "/api/joborders/copy/" + $row.data("dbid") + url_sid)
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Copy Job', {  category: 'Jobs', label: analyticsSource });
                        var $loadElement = $but;
                        if (_.isElement($row)) {
                            $loadElement = $row.find("[data-source=\"status\"]");
                        }
                        var load = PHD.showLoadingIndicator($loadElement);
                        PHD.Ajax.post($but.attr("href"), null, load)
                            .done(function (data) {
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-create-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Create Jobs");
                return true;
            }
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Create Jobs");
                return true;
            }
            return true;
        }

        if ($but.is(".btn-pause-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Pause Jobs");
                return true;
            }
            if ($row !== null && $row.length) { // lmc - not toggling system jobs. && !$row.is(".system-true")) {
                $but.removeClass("disabled")
                    .off("click");
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var analyticsEvent;
                if (selectedIDs.length === 1) {
                    if (_.isElement($row)) {
                        if ($row.is(".job-status-paused")) {
                            $but.attr("href", "/api/jobs/resume/" + $row.data("dbid") + url_sid)
                                .html('<i class="icon-play icon-large"></i>Resume');
                            analyticsEvent = "Resume Job";
                        } else if ($row.is(".job-status-queued")) {
                            $but.attr("href", "/api/jobs/suspend/" + $row.data("dbid") + url_sid)
                                .html('<i class="icon-pause icon-large"></i>Pause');
                            analyticsEvent = "Pause Job";
                        } else if ($row.is(".job-status-running") && $row.is(".job-type-certification")){
                            $but.attr("href", "/api/jobs/suspend/" + $row.data("dbid") + url_sid)
                                .html('<i class="icon-pause icon-large"></i>Pause');
                            analyticsEvent = "Pause Job";
                        }
                        else {
                            $but.disableButton();
                            return true;
                        }
                    } else {
                        if (selectedJob.status === 'Paused') {
                            $but.attr("href", "/api/jobs/resume/" + selectedJob.id + url_sid)
                                .html('<i class="icon-play icon-large"></i>Resume');
                            analyticsEvent = "Resume Job";
                        } else if (selectedJob.status === 'Queued') {
                            $but.attr("href", "/api/jobs/suspend/" + selectedJob.id + url_sid)
                                .html('<i class="icon-pause icon-large"></i>Pause');
                            analyticsEvent = "Pause Job";
                        } else if (selectedJob.status === 'Active' && selectedJob.type === 'certification') {
                            $but.attr("href", "/api/jobs/suspend/" + selectedJob.id + url_sid)
                                .html('<i class="icon-pause icon-large"></i>Pause');
                            analyticsEvent = "Pause Job";
                        }
                        else {
                            $but.disableButton();
                            return true;
                        }
                    }
                } else {
                    $but.attr("href", "/api/jobs/")
                        .html('<i class="icon-pause icon-large"></i> ' + gettextCatalog.getString('Toggle'));
                }

                $but.on("click", function (event) {
                    event.preventDefault();

                    var serviceCalled = false;

                    var suspendResumeCall = function(action, jobs) {
                        var jobsIds             = jobs.map(function(job) {return job.id;}).join(',');
                        var url                 = "/api/jobs/"+ action +"/" + jobsIds + "/?sid=" + jobs[0].system_id;
                        var loadSuspendResume   = PHD.showLoadingIndicator('body', true, 'Suspend/Resume Jobs....');

                        PHD.Ajax.put(url, null, loadSuspendResume)
                            .done(function(data) {
                                PHD.hideLoadingIndicator(loadSuspendResume);
                            })
                            .always(function(data) {
                                serviceCalled = true;
                                tab.clearSelectedRows();
                                tab.updateButtons();
                                tab.update(true);
                            });
                    }

                    var jobsToSuspend = selectedIDs.filter(function(job) {
                        return ! job.remote && isSuspendable(job.status, job.type);
                    });

                    var jobsToResume = selectedIDs.filter(function(job) {
                        return ! job.remote && isResumable(job.status);
                    });

                    if (jobsToSuspend.length) {
                        suspendResumeCall('suspend', jobsToSuspend);
                    }

                    if (jobsToResume.length) {
                        suspendResumeCall('resume', jobsToResume);
                    }
                });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-cancel-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Cancel Jobs");
                return true;
            }
            var finished = false;
            var canceled = false;
            if ($row !== null && selectedIDs.length === 1) {
                if (_.isElement($row)) {
                    finished = $row.is(".job-status-finished");
                    canceled = $row.is(".job-status-canceled");
                } else if (rowData !== null) {
                    finished = isFinished(rowData.status);
                    canceled = isCanceled(rowData.status);
                }
            }
            if ($row !== null && $row.length && !finished && !canceled) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var job_is_remote = selectedJob.remote;
                $but.removeClass("disabled")
                    .off("click")
                    .attr("href", "/api/jobs/" + selectedJob.id + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        var serviceCalled = false;
                        selectedIDs;

                        var cancelCall = function(jobs, payload = null) {
                            var jobsIds     = jobs.map(function(job) {return job.id;}).join(',');
                            var url         = "/api/jobs/" + jobsIds + "/?sid=" + jobs[0].system_id;
                            var loadCancel  = PHD.showLoadingIndicator('body', true, 'Cancelling Jobs....');

                            PHD.Ajax.delete(url, payload, loadCancel)
                            .done(function (data) {
                                PHD.hideLoadingIndicator(loadCancel);
                            })
                            .always(function(data) {
                                serviceCalled = true;
                                tab.clearSelectedRows();
                                tab.updateButtons();
                                tab.update(true);
                            })
                        }

                        var remoteJobsToCancel = selectedIDs.filter(function(job) {
                            return job.remote && ! isFinished(job.status) && ! isCanceled(job.status);
                        });

                        var localJobsToToCancel = selectedIDs.filter(function(job) {
                            return ! job.remote && ! isFinished(job.status) && ! isCanceled(job.status);
                        });

                        if (remoteJobsToCancel.length) {
                            remoteJobsToCancel.forEach(function(job) {
                                cancelCall(remoteJobsToCancel, JSON.stringify({remote:true}));
                            });
                        }

                        if (localJobsToToCancel.length) {
                            cancelCall(localJobsToToCancel);
                        }

                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-view-details")) {
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                $but.removeClass("disabled");
                $but.off("click");

                if (!tab.inventoryOpen) {
                    $but
                        .html('<i class="icon-tasks icon-large"></i>' + gettextCatalog.getString('View Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.showJobDetails();
                        });
                } else {
                    $but
                        .html('<i class="icon-eyeclose icon-large"></i>' + gettextCatalog.getString('Hide Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.hideJobDetails();
                        });
                }
            } else {
                $but.disableButton();
                $but.addClass("disabled");
            }
            return true;
        }

        if ($but.is(".btn-pop-out")) {
            if ($row !== null && $row.length && tab.index === JOB_TAB_HISTORY && rowData.status !== 'Queued') {
                $but.removeClass("disabled")
                    .off("click")
                    .on("click", function (event) {
                        event.preventDefault();
                        var sid = rowData.system_id;
                        var bids = rowData.backup_id;
                        var size = rowData.size;
                        var name = rowData.name;

                        if(rowData.type === "certification"){
                            if(rowData.status === "AlreadyRunning"){
                                var message = gettextCatalog.getString("Same job was already in progress");
                                PHD.throwNotice({"result": [{"message": message}]});
                            }
                            else{
                                var dcaSessionId =  parseInt(rowData.id.slice(0, rowData.id.indexOf(".")));
                                var url = "/api/certification/history/"+ dcaSessionId + "/?sid="+ sid + "&lang=" + $scope.userLanguageCode;
                                var load = PHD.showLoadingIndicator("body", true, "Loading...");
                                var resp = PHD.Ajax.get(url, load);
                                resp.done(function(data) {
                                    PHD.hideLoadingIndicator(load);
                                    var setTopvalue='top:75px;';
                                    if (screen.height >1024){
                                        setTopvalue='top:15%;'
                                    }else if(screen.height == 1024){
                                        setTopvalue='top:10%;'
                                    }
                                    var scope = angular.element(document.getElementById("idJobsPage")).scope();
                                    scope.DCAActiveJobDetails = false;
                                    var dimensions = 'width:1200px;height:500px;';
                                    if (data.history != undefined && data.history != null && data.history.detail != undefined && data.history.detail != null) {
                                        ngDialog.open({
                                            template: 'app/joborders/certify-backups/dcaJobDetails.html',
                                            modelDialogId: 'dcaJobDetails-dialog',
                                            name: 'dcaJobDetails-dialog',
                                            scope: scope,
                                            ngDialogPostionStyle: setTopvalue,
                                            ngDialogStyle: dimensions,
                                            data: data
                                        });
                                    } else {
                                        var message = gettextCatalog.getString("There are no details for this job.");
                                        PHD.throwNotice({"result": [{"message": message}]});
                                    }
                                });
                            }
                        }
                        else if (!isNaN(bids) && rowData.status != 'Cancelled') {
                            // These variables MUST BE set to use status.html.
                            PHD.appliance_sid = sid;
                            backup_ids = bids.toString();
                            copies = (rowData.type == 'replication');
                            ngDialog.open({
                                template: 'app/inventory/details/status.html',
                                scope: $scope,
                                overlay:true,
                                data: {'backup_ids': backup_ids, 'copies': copies, 'title': gettextCatalog.getString("Backup Statuses for Job:") + " " + name},
                                modelDialogId: 'backup-status-dialog',
                                name: 'backup-status-dialog',
                                ngDialogStyle: 'height:500px;width:1200px',
                                preCloseCallback: function (value) {
                                }
                            });
                        } else {
                            if (rowData.failed === false) {
                                copy_ids = rowData.id;
                                copies = true;
                                ngDialog.open({
                                    template: 'app/inventory/backup-copy-details/status.html',
                                    scope: $scope,
                                    overlay:true,
                                    data: {'copy_ids': copy_ids, 'copies': copies, 'title': gettextCatalog.getString("Backup Copy Statuses for Job:") + " " + name },
                                    modelDialogId: 'backup-copy-status-dialog',
                                    name: 'backup-copy-status-dialog',
                                    ngDialogStyle: 'height:400px;width:1200px',
                                    preCloseCallback: function (value) {
                                    }
                                });
                            } else {
                                var message = "";
                                message += (rowData.expanded_error !== undefined) ? rowData.expanded_error : rowData.message;
                                console.log("message " + rowData.message);
                                console.log("expanded_error " + rowData.expanded_error);
                                PHD.throwNotice({"result": [{"message": message}]});
                            }
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-disable-job")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot enable/disable jobs");
                return true;
            }
            $but.prop("title", "");
            if (selectedIDs.length > 0 || $row !== null && $row.length) {

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                var jobElement = angular.element(document.getElementById("idJobsPage"));
                var $jobScope = jobElement.scope();

                if (selectedIDs.length === 0) {
                    if (rowData.enabled === 1) {
                        $but.attr("href", "/api/joborders/disable/" + selectedJob.id + url_sid)
                            .html('<i class="icon-off icon-large"></i> ' + gettextCatalog.getString('Disable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = true;
                            if(selectedJob.status == 'InstantLab' || selectedJob.status == 'Failover') {
                                $jobScope.currentDCAJobIsFailover = true;
                            }
                        }
                    } else if (rowData.enabled === 0) {
                        $but.attr("href", "/api/joborders/enable/" + selectedJob.id + url_sid)
                            .html('<i class="icon-restart icon-large"></i> ' + gettextCatalog.getString('Enable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = false;
                            if(selectedJob.status == 'InstantLab' || selectedJob.status == 'Failover') {
                                $jobScope.currentDCAJobIsFailover = false;
                            }
                        }
                    }
                } else if (selectedIDs.length === 1) {
                    var selectedJob = selectedIDs[0];
                    var jobID = selectedJob.id;
                    var enabled = selectedJob.enabled;
                    if (enabled) {
                        $but.attr("href", "/api/joborders/disable/" + jobID + url_sid)
                            .html('<i class="icon-off icon-large"></i> ' + gettextCatalog.getString('Disable'));
                        if(selectedJob.type == "certification" ) {
                            $jobScope.currentDCAJobDisabled = true;
                            if(selectedJob.status == 'InstantLab' || selectedJob.status == 'Failover') {
                                $jobScope.currentDCAJobIsFailover = true;
                            }
                        }
                    } else {
                        $but.attr("href", "/api/joborders/enable/" + jobID + url_sid)
                            .html('<i class="icon-restart icon-large"></i> ' + gettextCatalog.getString('Enable'));
                        if(selectedJob.type == "certification" ) {
                            $jobScope.currentDCAJobDisabled = false;
                            if(selectedJob.status == 'InstantLab' || selectedJob.status == 'Failover') {
                                $jobScope.currentDCAJobIsFailover = false;
                            }
                        }
                    }
                    // don't allow selection if a single replication job.
                    if (selectedJob.type === 'replication') {
                        $but.disableButton();
                        $but.prop("title", "Hot Backup Copy must be disabled at the appliance-level");
                        return true;
                    }
                } else {
                    $but.attr("href", "/api/joborders/")
                        .html('<i class="icon-off icon-large"></i> ' + gettextCatalog.getString('Toggle'));
                    $but.prop("title", "Toggle enabled/disabled state of selected jobs")
                }
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        if (selectedIDs.length === 1) {
                            var $loadElement = $but;
                            if (_.isElement($row)) {
                                $loadElement = $row.find("[data-source=\"status\"]");
                            }
                            var load = PHD.showLoadingIndicator($loadElement);
                            $analytics && $analytics.eventTrack('Enable/Disable Job', {  category: 'Jobs', label: analyticsSource });
                            PHD.Ajax.put($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.update();
                                });
                        } else {
                            load = PHD.showLoadingIndicator('body', true, 'Enabling/Disabling Jobs....');
                            $analytics && $analytics.eventTrack('Enable/Disable Multiple Jobs', {  category: 'Jobs', label: analyticsSource });
                            var remainingCalls = selectedIDs.length;
                            var serviceCalled = false;
                            selectedIDs.forEach (function(jobObj, index) {
                                var type = jobObj.type;
                                if (type === 'replication') {
                                    // Don't disable replication jobs.
                                    --remainingCalls;
                                    if(remainingCalls <= 0 && !serviceCalled) {
                                        // If complete, and no service called, update and emove load indicator manually.
                                        tab.clearSelectedRows();
                                        tab.updateButtons();
                                        tab.update(true);
                                        setTimeout(function () {
                                            // Provide delay so the message will be displayed and then removed.
                                            PHD.hideLoadingIndicator(load);
                                        }, 500);
                                    }
                                    return true;
                                }
                                var id = jobObj.id;
                                var a = id.split('.');
                                var enabled = jobObj.enabled;
                                var toggleURL = $but.attr("href") + (enabled ? "disable" : "enable") + "/" + a[0] + "/?sid=" + a[1];
                                PHD.Ajax.put(toggleURL, null, load)
                                    .done(function (data) {
                                        ;
                                    })
                                    .always(function(data) {
                                        serviceCalled = true;
                                        --remainingCalls;
                                        if(remainingCalls <= 0) {
                                            // Once complete, uncheck all and update.
                                            tab.clearSelectedRows();
                                            tab.updateButtons();
                                            tab.update(true);
                                        }
                                    })
                            });
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-job")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Delete Jobs");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .attr("href", "/api/joborders/" + selectedJob.id + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        var formattedJobName = selectedJob !== null && selectedJob.name !== undefined ? (" " +selectedJob.name) : "";
                        $analytics && $analytics.eventTrack('Delete Job', {  category: 'Jobs', label: analyticsSource });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettextCatalog.getString("Are you sure you want to delete the job") + formattedJobName + "?</p>",
                                { title: gettextCatalog.getString("Confirm Job Deletion") }
                            );

                        confirm.done(function () {
                            var $loadElement = $but;
                            if (_.isElement($row)) {
                                $loadElement = $row.find("[data-source=\"status\"]");
                            }
                            var load = PHD.showLoadingIndicator($loadElement);
                            var jobID = selectedJob.id;
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    tab.deleteRow(jobID);
                                    tab.hideJobDetails();
                                    tab.update();
                                });
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-run-job")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot run jobs");
                return true;
            }
            if(rowData !== null && rowData.status !== "Idle"){
                $but.disableButton();
                return true;
            }
            if ($row !== null && $row.length && (rowData.enabled === 1) && (rowData.type !== 'replication') && selectedIDs.length < 2) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var url = selectedJob.type == "certification" ? "/api/joborders/test/" : "/api/joborders/run/";

                $but.removeClass("disabled")
                    .attr("href", url + selectedJob.id + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Run Job', {  category: 'Jobs', label: analyticsSource });
                        var $loadElement = $but;
                        if (_.isElement($row)) {
                            $loadElement = $row.find("[data-source=\"status\"]");
                        }
                        var load = PHD.showLoadingIndicator($loadElement);
                        PHD.Ajax.put($but.attr("href"), null, load)
                            .done(function (data) {
                                //PHD.hideLoadingIndicator(load);
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-run-jobs")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot run jobs");
                return true;
            }
        }

        if ($but.is(".btn-instantLab-job")) {
            if ($row !== null && $row.length && (rowData.enabled !== false) && (rowData.type === 'certification')) {
                var selectedJob = rowData;
                $but.removeClass("disabled");
            }
            else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-search-files") || $but.is(".btn-windows-recovery")){
            // monitor users cannot search files or start windows instant recovery
            if (!$scope.user.monitor){
                $but.removeClass("disabled");
            } else {
                var msg = "";
                if ($but.is(".btn-search-files")) {
                    msg = "Search Files";
                } else if ($but.is(".btn-windows-recovery")) {
                    msg = "start Windows Instant Recovery";
                }
                $but.addClass("disabled");
                $but.prop("title", "Monitor users cannot " + msg);
            }
        }

        if ($but.is(".btn-import-backup-copy")) {
            // monitor users cannot import backups
            if (!$scope.user.monitor){
                if ($row === null || !$row.length || !$row.is("[data-type=backup]") ||
                    ($row.filter("[data-type=backup]").length > 1) ||
                    !(($row.hasClass('backup-type-replication') && $row.data("remote") == true) || $row.hasClass('backup-type-archive'))) {
                    $but.disableButton();
                    return true;
                } else {
                    var selections_obj = tab.TreeTable.getSelectionObject();
                    var $archivedBackup = $row.filter("[data-type=backup]");
                    var instance = $archivedBackup.attr("instance-id");
                    if (instance === undefined) {
                        instance = $archivedBackup.attr("data-id");
                    }
                    selections_obj.remote = $archivedBackup.data("remote");
                    if (selections_obj.remote) {
                        var injector = elem.injector();
                        var rolesSvc = injector.get('rolesSvc');
                        if (rolesSvc.noDownloads($scope.user.role_recover_options)) {
                            $but.disableButton();
                            $but.prop("title", "This user cannot import backups from the hot backup copy target.");
                            return true;
                        }
                    }

                    $but.removeClass("disabled")
                        .on("click", function (event) {
                            event.preventDefault();
                            $analytics && $analytics.eventTrack('Import Copy', {  category: 'Recover', label: analyticsSource });
                            ngDialog.open({
                                template: 'app/recover/backup-copy/recover-backup-copy.html',
                                scope: $scope,
                                overlay:true,
                                data: {'selections': selections_obj, 'instance_id': instance },
                                modelDialogId: 'import-backup-copy-dialog',
                                name: 'import-backup-copy-dialog',
                                ngDialogStyle: 'height:590px;width:1200px;top:-40px',
                                preCloseCallback: function (value) {
                                }
                            });
                        });
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Import to Source");
                return true;
            }
        }

        if ($but.is(".btn-pin-backup")) {
            // monitor users cannot put backups on hold
            if (!$scope.user.monitor) {
                if ($scope.configureDisabled) {
                    $but.disableButton();
                    $but.prop("title", "Operators cannot hold backups");
                    return true;
                }
                if ($row === null || !$row.length ||
                    (!$row.is("[data-type=backup]") || ($row.is("[data-type=backup]") && $row.data("remote") == true)) ||
                    $row.filter('.backup-type-archive').length > 0 ||
                    $row.filter('.backup-type-imported').length > 0 ||
                    $row.filter("[data-remote=true]").length > 0 ||
                    $row.filter('.imported-from-target').length > 0) {
                    $but.prop("title", "Neither copied nor imported backups may be put on hold.");
                    $but.disableButton();
                    return true;
                } else {
                    $but.prop("title", "Backups on hold can never be deleted");
                    var daysToHold = 0;
                    $row = $row.filter("[data-type=backup]");

                    if ($row.is(".pin") && !$row.allAre(".pin")) {
                        $but.disableButton();
                        return true;
                    }
                    qs = "";

                    if ($row.is(".pin")) {
                        action = "unpin";
                        $but.html('<i class="icon-pin icon-large"></i> ' + gettextCatalog.getString('Unhold'));
                        daysToHold = 0;
                    } else if (!$row.is(".pin")) {
                        action = "pin";
                        $but.html('<i class="icon-pin icon-large"></i> ' + gettextCatalog.getString('Hold'));
                        daysToHold = -1;
                    }

                    $but.removeClass("disabled")
                        .attr("href", "/api/backups/hold/")
                        .on("click", function (event) {
                            event.preventDefault();
                            var selections = tab.TreeTable.getSelectionObject();
                            var remainingCalls = selections.backups.length;
                            selections.backups.forEach(function (backup_id, index) {
                                var sid = selections.system_ids.length > index ? selections.system_ids[index] : selections.system_id;
                                var url = $but.attr("href") + backup_id + "/" + daysToHold + "/?sid=" + sid;
                                var load = PHD.showLoadingIndicator($but);
                                PHD.Ajax.put(url, null, load)
                                    .done(function (data) {
                                        tab.pinRows(backup_id, action);
                                    })
                                    .always(function (data) {
                                        --remainingCalls;
                                    })
                            });
                            if (remainingCalls <= 0) {
                                tab.updateButtons();
                                tab.update(true);
                            }
                        });
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Hold backups");
                return true;
            }
        }

        if ($but.is(".btn-test-dr")){
            if (!$scope.user.monitor){
                $but.removeClass("disabled");
            } else {
                $but.prop("title", "Monitor users cannot request a DR Test")
            }
        }

        if ($but.is(".btn-manage-backups")){
            if (!$scope.user.monitor){
                $but.removeClass("disabled");
            } else {
                $but.prop("title", "Monitor users cannot manage backups")
            }
        }

        if ($but.is(".btn-delete-backup")) {
            // monitor users cannot delete backups
            if (!$scope.user.monitor) {
                if ($scope.configureDisabled) {
                    $but.disableButton();
                    $but.prop("title", "Operators cannot delete backups");
                    return true;
                }
                if ($row !== null && $row.length && $row.is("[data-type=backup]")) {
                    $but.removeClass("disabled")
                        .attr("href", "/api/backups/related/")
                        .on("click", function (event) {
                            event.preventDefault();
                            var remoteBackupsChecked = $row.filter("[data-remote=true]").length > 0;
                            if (!$row.hasClass('backup-type-archive') && !remoteBackupsChecked) {
                                var load = PHD.showLoadingIndicator($but);
                                var related_url;
                                var selections = tab.TreeTable.getSelectionObject();
                                var bid_string = selections.backups.toString();
                                related_url = $but.attr("href") + bid_string + "/?sid=" + selections.system_id + '&original&lang=' + $scope.userLanguageCode;
                                PHD.Ajax.get(related_url, load)
                                    .done(function (data) {
                                        $analytics && $analytics.eventTrack('Delete Backup', {
                                            category: 'Backup',
                                            label: analyticsSource
                                        });
                                        tab.confirmDelete(data);
                                    });
                            } else if ($row.hasClass('backup-type-archive')) {
                                var message = "Individual Backup Copies cannot be deleted." + "<br/>" + "<br/>" +
                                    "Go to the appliance configuration page for the target to purge all backup copies.";
                                PHD.throwNotice({"result": [{"message": message}]}, {});
                            } else if (remoteBackupsChecked) {
                                var message = "Hot Backup Copies on the target cannot be deleted.";
                                PHD.throwNotice({"result": [{"message": message}]}, {});
                            }
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Delete backups");
                return true;
            }
        }

        if ($but.is(".btn-flr-details")) {
            // monitor users cannot view details
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length) {
                    $but.removeClass("disabled")
                        .attr("href", "/api/jobs/active/flr/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.showFLRDetails($row.attr("flr_id"), $row.attr('system-id'));
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot view details");
                return true;
            }
        }

		/* if ($but.is("#id_ir_btn_live")) {
             // monitor user cannot set WIR in live mode
             if (!$scope.user.monitor) {
                 if ($row !== null && $row.length) {
                     if (jobType === "wir") {
                         if (wirMode === gettextCatalog.getString(MODE_NEW) ||
                             wirMode === gettextCatalog.getString(MODE_HALTED) ||
                             wirMode === gettextCatalog.getString(MODE_AUDIT) ||
                             wirMode === gettextCatalog.getString("stopAudit")) {
                             $but.disableButton();
                         } else {
                             if (wirMode === gettextCatalog.getString(MODE_LIVE)) {
                                 $but.html('<i class="icon-binoculars icon-large"></i> ' + gettextCatalog.getString('End Live'));
                                 $but.removeClass("disabled")
                                     .attr("href", "/api/virtual_clients/live/stop/" + virtual_id + "/?sid=" + sid)
                                     .on("click", function (event) {
                                         event.preventDefault();
                                         var liveUrl = $but.attr("href");
                                         PHD.Ajax.put(liveUrl)
                                             .done(function (data) {
                                                 wirMode = gettextCatalog("stopLive");
                                                 tab.update();
                                             })
                                     });
                             } else {
                                 $but.html('<i class="icon-binoculars icon-large"></i> ' + gettextCatalog.getString('Go Live'));
                                 $but.removeClass("disabled")
                                     .attr("href", "/api/virtual_clients/live/start/" + virtual_id + "/?sid=" + sid)
                                     .on("click", function (event) {
                                         event.preventDefault();
                                         var liveUrl = $but.attr("href"),
                                             confirm = PHD.confirmDialog(
                                                 "<p>" + gettextCatalog.getString("Once you go live, any future backups will not be restored to this Windows replica, as it will be marked invalid.") +
                                                 "<br/><br/>" + gettextCatalog.getString("Are you sure you wish to continue?") + "</p>",
                                                 {title: gettextCatalog.getString("Confirm Go Live")}
                                             );
                                         confirm.done(function () {
                                             PHD.Ajax.put(liveUrl)
                                                 .done(function (data) {
                                                     wirMode = gettextCatalog.getString(MODE_LIVE);
                                                     tab.update();
                                                 })
                                         });

                                     });
                             }
                         }
                     } else if (jobType === "VMware host"){
                         console.log("buttonlogic job type should be vmware" + jobType);
                         $but.prop("title", "Please click View Details and select a recovery point from the Recovery Points tab");
                     } else {
                         $but.disableButton();
                     }
                 } else {
                     $but.disableButton();
                 }
                 return true;
             } else {
                 $but.disableButton();
                 $but.prop("title", "Monitor users cannot set Instant Recovery to go into live mode");
                 return true;
             }
        }

        if ($but.is("#id_ir_btn_audit")) {
            // monitor users cannot set WIR in audit mode
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length) {
                    if (jobType == "wir") {
                        if (wirMode === gettextCatalog.getString(MODE_NEW) ||
                            wirMode === gettextCatalog.getString(MODE_HALTED) ||
                            wirMode === gettextCatalog.getString(MODE_LIVE) ||
                            liveTime !== "" ||
                            wirMode == gettextCatalog("stopLive")) {
                            $but.disableButton();
                        } else {
                            if (wirMode === gettextCatalog.getString(MODE_AUDIT)) {
                                $but.html('<i class="icon-rawaccesslogs icon-large"></i> ' + gettextCatalog.getString('End Audit'));
                                $but.removeClass("disabled")
                                    .attr("href", "/api/virtual_clients/audit/stop/" + virtual_id + "/?sid=" + sid)
                                    .on("click", function (event) {
                                        event.preventDefault();
                                        var auditUrl = $but.attr("href");
                                        PHD.Ajax.put(auditUrl)
                                            .done(function (data) {
                                                wirMode = gettextCatalog.getString("stopAudit");
                                                tab.update();
                                            });
                                    });
                            } else {
                                $but.html('<i class="icon-rawaccesslogs icon-large"></i> ' + gettextCatalog.getString('Audit'));
                                $but.removeClass("disabled")
                                    .attr("href", "/api/virtual_clients/audit/start/" + virtual_id + "/?sid=" + sid)
                                    .on("click", function (event) {
                                        event.preventDefault();
                                        var auditUrl = $but.attr("href"),
                                            confirm = PHD.confirmDialog(
                                                "<p>" + gettextCatalog.getString("Are you sure you want to go to audit mode?") + "</p>",
                                                {title: gettextCatalog.getString("Confirm Audit")}
                                            );
                                        confirm.done(function () {
                                            PHD.Ajax.put(auditUrl)
                                                .done(function (data) {
                                                    wirMode = gettextCataolog.getString(MODE_AUDIT);
                                                    tab.update();
                                                });
                                        });
                                    });
                            }
                        }
                    } else if (jobType === "VMware host"){
                        $but.prop("title", "Please click View Details and select a recovery point from the Recovery Points tab");
                    } else {
                        $but.disableButton();
                    }
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot set Instant Recovery to go into audit mode");
                return true;
            }
        }*/

        if ($but.is(".btn-delete-flr")) {
            //monitor users cannot delete FLR objects
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length) {
                    var flr_id = $row.attr("flr_id");
                    var flr_is_remote = $row.data("remote");
                    $but.removeClass("disabled")
                        .attr("href", "/api/jobs/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id') + "&host=" + $row.attr('data-host'))
                        .on("click", function (event) {
                            event.preventDefault();
                            $analytics && $analytics.eventTrack('Delete FLR', {
                                category: 'Recover',
                                label: analyticsSource
                            });

                            var url = $but.attr("href"),
                                confirm = PHD.confirmDialog(
                                    "<p>" + gettextCatalog.getString("Are you sure you want to delete this FLR Object?") + "</p>",
                                    {title: gettextCatalog.getString("Confirm FLR Deletion")}
                                );

                            confirm.done(function () {
                                var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                                $row.find("[data-source=\"status\"]").html(gettextCatalog.getString("Deleting"));
                                var payload = null;
                                if (flr_is_remote === true) {
                                    payload = JSON.stringify({remote: true});
                                }
                                PHD.Ajax.delete(url, payload, load)
                                    .done(function (data) {
                                        //PHD.hideLoadingIndicator(load);
                                        tab.deleteFLR(flr_id);
                                    });
                            });
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot delete FLR");
                return true;
            }
        }

        //if ($but.is(".btn-instant-restore")) {
        //    if ($row !== null && $row.length && ($row.is(".hv-type-xen") || $row.hasClass('backup-type-archive')
        //        || $row.attr('app-type') === APPLICATION_TYPE_NAME_PHYSICAL_SERVER)) {
        //        $but.disableButton();
        //        return true;
        //    }
        //    var catalogView = $but.data("catalog-view") || BACKUP_CATALOG_VIEW_VM;
        //    var selections_obj = getSelectedBackups(tab);
        //
        //    $but.removeClass("disabled")
        //        .attr("href", "app/joborders/recover/recover.html")
        //        .on("click", function (event) {
        //            event.preventDefault();
        //            $analytics && $analytics.eventTrack('Recover Instant', {  category: 'Recover', label: analyticsSource });
        //            PHD.wizard($but.attr("href"), "instant-recovery", {
        //                title: gettext("Create Instant Recovery Job"),
        //                helpArticle: PHD.App.getHelpLink("instantrestore", true),
        //                dialogClass: "dialog-ir dialog-modal-form",
        //                // lmc
        //                open: function (event, ui) {
        //                    //$("#id_name").val("Instant Recovery Job");
        //                    // Recover Jobs are only on-demand and show as such in the active jobs list.
        //                    $("#id_name").val("On-Demand");
        //                    $("#id_name").prop("disabled", true);
        //                    $("#full_show_time").remove();
        //                    jobEdit = false;
        //                    isIRJob = true;
        //                    selections = selections_obj;
        //                    backupCatalogView = catalogView;
        //                }
        //            });
        //        });
        //    return true;
        //}
/*
        if ($but.is(".btn-ir-end")) {
            if ($row !== null && $row.length && !$row.is(".instant-state-ending")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .attr("href", "/api/jobs/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Delete IR', {  category: 'Recover', label: 'IR' });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettext("Are you sure you want to stop this IR session?") + "<br/><br/>" +
                                        gettext("The recovery object will be removed from the appliance, and if the VM was created ") +
                                        gettext("in audit mode, it will be deleted from the hypervisor.") + "</p>",
                                { title: gettext("Confirm IR Tear Down") }
                            );

                        confirm.done(function () {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            $row.find("[data-source=\"status\"]").html("Deleting");
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    // upon deletion, hide details.
                                    tab.hideJobDetails();
                                    tab.update();
                                });
                        });
                    })
            } else {
                $but.disableButton();
            }
            return true;
        }*/

        if ($but.is(".btn-dca")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Create Jobs");
                return true;
            }
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Create Jobs");
                return true;
            }
            return true;
        }

        if ($but.is(".btn-archive")) {
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Create Jobs");
                return true;
            }
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Create Jobs");
                return true;
            }
            return true;
        }

        if ($but.is(".btn-export-job-history") || $but.is(".btn-export-ir-history") || $but.is(".btn-export-sync-history")) {
            $but.attr("href");
            return true;
        }

        if ($but.is(".btn-export-csv")) {
            $but.attr("href", tab.getUpdateUrl(true));
            return true;
        }

        if ($but.is(".btn-save-log")) {
            if ($row !== null && $row.length) {
            }
            return true;
        }

        if ($but.is(".btn-show-filter")) {
            $but.on("click", function (event) {
                event.preventDefault();
                if (tab) {
                    tab.showFilter();
                }
            });
            return true;
        }

        if ($but.is(".btn-manage-retention")) {
            $but.on("click", function (event) {
                event.preventDefault();
                $(document).trigger("launchdialog", {"action": "retention"});
            });
        }

        if ($but.is(".btn-backup-details")) {
            $but.prop("title", "");
            // monitor users cannot view details
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length > 0) {
                    if ($row.hasClass('backup-type-archive')) {
                        $but.disableButton();
                        $but.prop("title", "Click on Import to Source to view Cold Backup Copy Status");
                        return true;
                    }
                    if ($row.length > 500) {
                        $but.disableButton();
                        $but.prop("title", "Please de-select some backups to view details.");
                        return true;
                    }
                    if ($row.filter("[data-remote=true]").length > 0) {
                        $but.disableButton();
                        $but.prop("title", "Cannot get detailed status of remote backups on the target.");
                        return true;
                    }
                    var hasImported = $row.hasClass('backup-type-imported');
                    $but.removeClass("disabled")
                        .on("click", function (event) {
                            event.preventDefault();
                            var selections = tab.TreeTable.getSelectionObject();
                            if (_.uniq(selections.system_ids).length > 1) {
                                // Details of backups from more than one system specified. Not allowed.
                                ngDialog.open({
                                    dialogType: 'ERROR',
                                    dialogMessage: gettextCatalog.getString('When viewing status details, all selections must be from the same appliance.'),
                                    overlay:true
                                });
                                $but.disableButton();
                                return true;
                            }
                            var bids = selections.backups;
                            var sid = selections.system_ids[selections.system_ids.length - 1];
                            // Old dialog for details display.
                            if ($scope.dialogOpen === false || $scope.dialogOpen === undefined) {
                                $scope.dialogOpen = true;
                                // These variables MUST BE set to use status.html.
                                PHD.appliance_sid = sid;
                                backup_ids = bids.toString();
                                copies = false;
                                ngDialog.open({
                                    template: 'app/inventory/details/status.html',
                                    scope: $scope,
                                    overlay: true,
                                    data: {
                                        'backup_ids': backup_ids, 'copies': copies,
                                        'title': gettextCatalog.getString("Backup Status Details"),
                                        showActions: false, showAssetName: true, hasImported: hasImported
                                    },
                                    modelDialogId: 'backup-status-dialog',
                                    name: 'backup-status-dialog',
                                    ngDialogStyle: 'height:500px;width:1200px',
                                    preCloseCallback: function (value) {
                                        $scope.dialogOpen = false;
                                    }
                                });
                            }
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot view details");
                return true;
            }
        }

        if ($but.parent().is(".btn-group")) {
            $but
                .off("click.btngroup")
                .on("click.btngroup", function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    var $menu = $but.parent().toggleClass("open");

                    if ($menu.is(".open")) {
                        $('body').one("click.btngroupoff", function (event) {
                            event.preventDefault();
                            $menu.removeClass("open");
                        });
                    }
                })
                .next("ul").find("a")
                .off("click.btngroup")
                .on("click.btngroup", function () {
                    event.preventDefault();
                    $but.parent().removeClass("open");
                });
            return true;
        }

        $but.on("click", function (event) {
            if (!$but.is(".plain-link") && !$but.closest(".note").length)
                event.preventDefault();
        });

        return false;
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
