(function($, PHD, window, undefined) {

    var defaultTVBuilerOptions = {
        hypervisorView: DEFAULT_VMWARE_VIEW,
        disks: 1,
        checkboxes: true,
        isCopy: 0,
        loader: null,
        sid: -1
    };

    var DestinationTreeViewBuilder = function DestinationTreeViewBuilder(options) {
        var self = this,
            //worker = new Worker('scripts/phd.treeview-worker.js?' + Date.now()),
            worker = new Worker('scripts/utils/treeview-worker.js?' + Date.now()),
            promise = $.Deferred(),
            url;

        this.options = $.extend({}, defaultTVBuilerOptions, options);

        url = "/" + this.options.slug + "/" + this.options.environment + "/";

        if(this.options.backup) url += "?backup=" + this.options.backup;

        resp = PHD.Ajax.get(url);

        worker.addEventListener('message', function(e) {
            promise.resolve(e.data);
        }, false);

        resp.done(function(data) {
            data.checkboxes = self.options.checkboxes;
            worker.postMessage(data);
        });

        return promise;
    };

    PHD.DestinationTreeViewBuilder = DestinationTreeViewBuilder;

    var TreeViewBuilder = function TreeViewBuilder(options) {
        var self = this,
            worker = new Worker('scripts/utils/treeview-worker.js?' + Date.now()),
            promise = $.Deferred(),
            url = "/api/inventory/", hvstring, resp;

        this.options = $.extend({}, defaultTVBuilerOptions, options);

        url += "?disks=" + this.options.disks;
        url += "&grandclient=" + this.options.isCopy;
        if (this.options.isCopy === 0 && this.options.sid !== -1) {
            url += "&sid=" + this.options.sid;
        }

        // lmc - TODO research this as I do not believe we will support HV view since we don't always have hosts.
        if($.isArray(this.options.hypervisorView)) {
            hvstring = this.options.hypervisorView.map(function(value, index) {
                return "&hypervisor=" + value;
            }).join("");
        } else {
            hvstring = "&hypervisor=" + this.options.hypervisorView;
        }

        if (this.options.uid !== undefined) {
            // pass user id for navigational views.
            hvstring = "&uid=" + this.options.uid;
        }

        // get the loading indicator if set.
        var loader = this.options.loader;

        resp = PHD.Ajax.get(url + hvstring, loader);

        worker.addEventListener('message', function(e) {
            promise.resolve(e.data);
        }, false);

        resp.done(function(data) {
            data.checkboxes = self.options.checkboxes;
            worker.postMessage(data);
        });

        return promise;
    };

    PHD.TreeViewBuilder = TreeViewBuilder;
    
    var TreeView = function TreeView($treeContainer, $treeSelections) {
        if(this instanceof TreeView) {
            this.$container = $treeContainer; // .treeview
            this.$selections = $treeSelections; // .treeview-selections
            this.$childContainer = this.$container.find(".children").first();
            this.states = [
                "desc-modified", 
                "expl-included", 
                "expl-excluded", 
                "impl-included", 
                "impl-excluded"
            ];

            this.isLoading = false;
        } else {
            return new TreeView($treeContainer, $treeSelections).init();
        }
    };

    TreeView.prototype = {
        init: function() {
            var self = this,
                $tree = this.$container;

            this.$universe = this.$container.find(".tree-universe").first();
            this.universeID = this.$universe.data("dbid");
            this.hasDiskCheckboxes = true;

            // Set up DOM event handlers
            $tree
                .on("click", ".header", function(event) {
                    var $header = $(this),
                        $children = $header.nextAll(".children").toggleClass("open");

                    // Toggle the treeview item's plus/minus button
                    $header.find(".expand-ctrl").toggleClass("expanded");
                    $children.trigger("expand");

                    return false;
                })
                .on("click", ".checkbox", function(event) {
                    event.stopPropagation();
                    self.handleCheckboxClick($(this).closest(".checkable"), false, false);

                    // Trigger an event for other scripts to consume
                    $(this).trigger("checkclick", [$(this).closest(".checkable"), false]);
                });

            return this;
        },

        setDiskCheckboxes: function(value) {
            this.hasDiskCheckboxes = value === true ? true : false;
            return this;
        },

        initSelections: function(simulateClick, appendId) {
            var self = this;
            this.setLoading(true);

            this.$selections.find('input[name="inventory"]').each(function() {
                var $node = self.$container.find("[data-dbid=\"" + $(this).val() + "\"]");
                self.handleCheckboxClick($node, true, false);
                
                if(simulateClick === true) {
                    $node.children(".header").children(".checkbox").trigger("checkclick", [$node, simulateClick]);
                }
            });

            this.$selections.find('input[name="inventory_exclude"]').each(function() {
                var $node = self.$container.find("[data-dbid=\"" + $(this).val() + "\"]");
                self.handleCheckboxClick($node, false, true);
            });

            if(appendId === true) {
                this.$container.find(".item-name").each(function() {
                    $(this).append(" <strong>" + $(this).closest(".checkable").data("dbid") + "</strong>");
                });
            }

            this.setLoading(false);
            return this;
        },

        isInheritedState: function($node) {
            return !$node.hasClass("expl-included") && !$node.hasClass("expl-excluded");
        },

        isBlank: function($node) {
            var states = this.states.map(function(val) {
                return "." + val;
            });
            return !$node.is(states.join(","));
        },

        isDuplicate: function($node) {
            var $duplicates = this.$container.find("[data-dbid=\"" + $node.data("dbid") + "\"]");
            return $duplicates.length > 1 ? $duplicates : false;
        },

        handleCheckboxClick: function($node, forceCheck, forceExclude, skipDupeCheck) {
            var self = this,
                nodeType = $node.data("node"),
                $selection = $node,
                $duplicates = this.isDuplicate($node);

            if($duplicates && !skipDupeCheck === true) {
                $duplicates.each(function() {
                    self.handleCheckboxClick($(this), false, false, true);
                });
                return this;
            }

            if(this.isInheritedState($selection) && !$selection.is("[data-node=disk]") && forceExclude !== true || forceCheck) {
                return this.includeNode($selection).handleAncestors($selection);
            }

            if($selection.hasClass("expl-included") || forceExclude || 
                    $selection.is("[data-node=disk]") && this.isInheritedState($selection)) {
                return this.excludeNode($selection).handleAncestors($selection);
            }

            if($selection.hasClass("expl-excluded")) {
                return this.setInheritedState($selection);
            }
        },

        setState: function($node, state) {
            var toRemove = this.states.join(" ").replace(state, "");
            $node.removeClass(toRemove).addClass(state);
            return this;
        },

        setInheritedState: function($node) {
            this.removeSelection($node, "_exclude");
            if($node.is(".tree-universe") || $node.data("dbid") == 1) {
                var $explChildren = $node.find(".expl-included, .expl-excluded");
                this.setState($node, "").removeChildSelections($node);
                if($explChildren.length) {
                    var self = this;
                    $explChildren.each(function() { self.handleAncestors($(this)) });
                }
                return this;
            } else {
                var $parent = this.getParentNode($node);
                if($parent.hasClass("expl-included") || $parent.hasClass("impl-included")) {
                    this.setState($node, "impl-included")
                        .setAffectedDescendants($node, "impl-included");
                } else if($parent.hasClass("expl-excluded") || $parent.hasClass("impl-excluded")) {
                    this.setState($node, "impl-excluded")
                        .setAffectedDescendants($node, "impl-excluded");
                } else if($parent.hasClass("desc-modified") && $node.find(".expl-included, .expl-excluded").length) {
                    this.setState($node, "desc-modified")
                        .setAffectedDescendants($node, "").handleDescendantAncestors($node);
                } else {
                    this.setState($node, "").removeChildSelections($node);
                }
            }
            return this.handleAncestors($node);
        },

        setLoading: function(isLoading) {
            this.isLoading = isLoading === true ? true : false;
            return this;
        },

        hasSelection: function(id, control) {
            return $('[name="' + control + '"][value="' + id + '"]', this.$selections).length > 0;
        },

        removeSelection: function($node, extra) {
            if (!this.isLoading) {
                var dbid = $node.data("dbid"),
                    controlName = this.$container.data("name") + extra;

                if (dbid) {
                    $('[name="' + controlName + '"][value="' + dbid + '"]').remove();
                }
            }
            return this;
        },

        checkAncestors: function($node, classToCheck) {
            return $node.parentsUntil(this.$container, ".checkable").is("." + classToCheck);
        },

        includeNode: function($node) {
            if ($node.hasClass("expl-excluded")) {
                this.removeSelection($node, '_exclude[]');
            }

            this.setState($node, "expl-included");

            this.addSelection($node, '').implicitlyIncludeChildren($node);
            return this;
        },

        addSelection: function($node, extra) {
            if (!this.isLoading) {
                var dbid = $node.data("dbid"),
                    controlName = this.$container.data("name") + extra;

                if (!this.hasSelection(dbid, controlName)) {
                    this.$selections.append('<input type="hidden" value="' + dbid + '" name="' + controlName + '">');
                    //console.log(this.$selections.html());
                    // TODO: Correct this to hide the actual error message
                    $('label[for="' + this.$container.data("validator") + '"]').css("display", "none");
                }
            }
            return this;
        },

        hasSelections: function() {
            return this.$selections.find("input").not("[value=" + this.universeID + "]").length > 0;
        },

        clearSelections: function(universe) {
            var $toDelete = this.$selections.find("input");
            if(universe === false) {
                $toDelete = $toDelete.not("[value=" + this.universeID + "]");
            }
            $toDelete.remove();
            return this;
        },

        implicitlyIncludeChildren: function($node) {
            var $children = this.getAffectedDescendants($node);

            $children
                .not(".expl-included, .expl-excluded")
                .attr("class", "checkable impl-included");

            return this;
        },

        excludeNode: function($node) {
            if ($node.hasClass("expl-included")) {
                this.removeSelection($node, '');
            }

            this.setState($node, "expl-excluded");
            // lmc add '[]' so that submitData picks up all Exclusions.
            this.addSelection($node, '_exclude[]').implicitlyExcludeChildren($node);
            return this;
        },

        implicitlyExcludeChildren: function($node) {
            var $children = this.getAffectedDescendants($node);

            $children
                .not(".expl-included, .expl-excluded")
                .attr("class", "checkable impl-excluded");

            return this;
        },

        removeChildSelections: function($node) {
            var $children = this.getAffectedDescendants($node);

            $children
                .not(".expl-included, .expl-excluded")
                .attr("class", "checkable");
                
            return this;
        },

        getAffectedDescendants: function($node) {
            var $children = $node.find(".checkable"),
                $ignore = $();

            $children.each(function() {
                var $child = $(this);
                if($child.hasClass("expl-excluded") || $child.hasClass("expl-included")) {
                    $ignore = $ignore.add($child.find(".checkable"));
                }
            });

            $children = $children.not($ignore).not(".expl-included, .expl-excluded");

            return $children;
        },

        setAffectedDescendants: function($node, className) {
            $children = this.getAffectedDescendants($node);
            $children.attr("class", "checkable " + className);
            return this;
        },

        excludeDisks: function($vm, $except) {
            var self = this;
            var $disks = $vm.find(".checkable").not($except);
            $disks.each(function() {
                self.excludeNode($(this));
            });
            return this;
        },

        handleAncestors: function($node) {
            var $parents = $node.parentsUntil(this.$container, ".checkable")
                    .not(".expl-included, .expl-excluded, .impl-included, .impl-excluded"),
                someSelected = false,
                self = this;

            $parents.each(function() {
                if(!someSelected) {
                    if($(this).find(".expl-excluded, .expl-included").length) {
                        someSelected = true;
                    }
                }

                if(someSelected) {
                    self.setState($(this), "desc-modified");
                } else {
                    self.setState($(this), "");
                }
            });
            return this;
        },

        handleDescendantAncestors: function($node) {
            var self = this;
            $node.find(".expl-included, .expl-excluded").each(function() {
                self.handleAncestors($(this));
            });
            return this;
        },

        expandAll: function() {
            //var start = Date.now(), end;
            this.$container.find("[data-node=container] > .header .expand-ctrl").addClass("expanded");
            this.$container.find("[data-node=container] > .children").addClass("open");
            //end = Date.now();
            //console.log("time to expand: " + (end - start));
        },
        collapseAll: function() {
            //var start = Date.now(), end;
            var collapseTarget = this.$container.children().children();
            collapseTarget.find("[data-node=container] > .header .expand-ctrl").removeClass("expanded");
            collapseTarget.find("[data-node=container] > .children").removeClass("open");
            //end = Date.now();
            //console.log("time to collapse: " + (end - start));
        },
        getNode: function(id) {
            return this.$container.find(".checkable[data-dbid=\"" + id + "\"]");
        },
        getNodeType: function(id) {
            return this.getNode(id).data("node");
        },
        getNodeName: function(id) {
            return this.getNode(id).children(".header").children(".item-name").html();
        },
        getParentNode: function($node) {
            return $node.parent().closest(".checkable");
        }

    };

    PHD.TreeView = TreeView;

    var ProtectTreeView = function($treeContainer, $treeSelections) {
        if(this instanceof ProtectTreeView) {
            this.$container = $treeContainer; // .treeview
            this.$selections = $treeSelections; // .treeview-selections
            this.$childContainer = this.$container.find(".children").first();
            this.states = [
                "desc-modified", 
                "expl-included", 
                "expl-excluded", 
                "impl-included", 
                "impl-excluded"
            ];

            this.isLoading = false;
            //this.hasSelections = false;
        } else {
            return new ProtectTreeView($treeContainer, $treeSelections).init();
        }
    };

    ProtectTreeView.prototype = Object.create(TreeView.prototype);

    ProtectTreeView.prototype.init = function() {
            var self = this,
                $tree = this.$container;

            this.$universe = this.$container.find(".tree-universe").first();
            this.universeID = this.$universe.data("dbid");
            this.selectedID = null;

            this.hasDiskCheckboxes = true;
            this.$selector = $("<i/>", {
                "class": "icon-circle-arrow-right icon-large selector"
            });

            // Set up DOM event handlers
            $tree
                .on("click", ".header", function(event) {
                    var $header = $(this),
                        $children = $header.nextAll(".children");

                    $node = $header.parent();
                    if(self.selectedID != $node.data("dbid")) {
                        $tree.find(".selected").removeClass("selected");
                        $header.addClass("selected");
                        self.selectedID = $node.data("dbid");
                        $node.trigger("nodeclick", $node.data("dbid"));
                    } else {
                        // Toggle the treeview item's plus/minus button
                        $header.find(".expand-ctrl").toggleClass("expanded");
                        $children.toggleClass("open")
                    }

                    return false;
                })
                .on("click", ".expand-ctrl", function(event) {
                    var $ctrl = $(this),
                        $children = $ctrl.parent().nextAll(".children").toggleClass("open");

                    event.stopPropagation();

                    // Toggle the treeview item's plus/minus button
                    $ctrl.toggleClass("expanded");

                    return false;
                })
                .on("click", ".selector", function(event) {
                    event.stopPropagation();
                })
                .on("click", ".checkbox", function(event) {
                    event.stopPropagation();
                    self.handleCheckboxClick($(this).closest(".checkable"), false, false);

                    // Trigger an event for other scripts to consume
                    $(this).trigger("checkclick");
                });

            return this;
    };

    ProtectTreeView.prototype.clearSelections = function() {
        this.$container.find(".selected").removeClass("selected");
        this.selectedID = null;
        return this;
    };

    PHD.ProtectTreeView = ProtectTreeView;

    var TreeViewSearch = function TreeViewSearch($controls, treeview, $sb, $sc) {
        if(this instanceof TreeViewSearch) {
            this.$controls = $controls;
            this.treeview = treeview;
            this.$treeview = treeview.$container;
            this.$scrollBox = $sb;
            this.$scrollContent = $sc;
            this.$buttons = this.$controls.find(".treeview-find-buttons")

            this.searchText = "";

            this.$results = null;
            this.$current = null;
            this.currentIndex = 0;
        } else {
            return new TreeViewSearch($controls, treeview, $sb, $sc).init();
        }
    };

    TreeViewSearch.prototype = {
        init: function() {
            //console.log("initializing treeview search");
            var self = this;
            this.$searchTextBox = this.$controls.find(".tree-search-text");
            this.$searchFindBut = this.$controls.find(".tree-search-find");
            this.$searchNextBut = this.$controls.find(".tree-search-next");
            this.$searchPrevBut = this.$controls.find(".tree-search-prev");
            this.$searchClearBut = this.$controls.find(".tree-search-clear");
            this.$searchMatchLabel = this.$controls.find(".tree-search-matches");

            this.initButtons();

            this.$searchTextBox.on("keydown", function(event) {
                event.stopPropagation();
                switch ( event.keyCode ) {
                    case $.ui.keyCode.TAB:
                        if(self.$results) {
                            event.preventDefault();
                            if(event.shiftKey) {
                                self.$searchPrevBut.click();
                                break;
                            }
                            self.$searchNextBut.click();
                        }
                        break;
                    case $.ui.keyCode.ESCAPE:
                        event.preventDefault();
                        self.$searchClearBut.click();
                        break;
                    case $.ui.keyCode.ENTER:
                        event.preventDefault();
                        self.$searchFindBut.click();
                        break;
                }
            });

            return this;
        },

        initButtons: function() {
            var self = this;
            this.$searchFindBut.on("click", function() {
                self.search();
            });
            this.$searchNextBut.on("click", function() {
                self.next();
            }).addClass("hidden");
            this.$searchPrevBut.on("click", function() {
                self.previous();
            }).addClass("hidden");
            this.$searchClearBut.on("click", function() {
                self.clear();
            }).addClass("hidden");
            this.updateButtons();
        },

        setButton: function($button, state) {
            if(state === "show") {
                $button.removeClass("hidden");
            } else if(state === "hide") {
                $button.addClass("hidden");
            }
            this.updateButtons();
        },

        updateButtons: function() {
            this.$buttons.find("button").removeClass("last").not(".hidden").last().addClass("last");
        },

        search: function() {
            this.searchText = $.trim(this.$searchTextBox.val());

            this.clear();

            if (this.searchText !== "") {
                this.$searchTextBox.val(this.searchText);
                this.$results = this.$treeview.find('.item-name:icontains("' + this.searchText + '")');
                var len = this.$results.length;

                this.$searchMatchLabel.show().find("span").text(len);

                if (len > 0) {
                    this.$results.addClass("search-found");

                    this.$current = this.$results.eq(this.currentIndex);

                    this.expandToMatch(this.$current.addClass("search-current"));

                    this.setButton(this.$searchFindBut, "show");
                    this.setButton(this.$searchClearBut, "show");
                    if(len > 1) {
                        this.setButton(this.$searchPrevBut, "show");
                        this.setButton(this.$searchNextBut, "show");
                    }
                }
                else {
                    this.$results = null;
                    // removing this to be consistent with Backup inventory search.
                    //PHD.alert(gettext("Nothing in your inventory matched your search text."));
                }
            }
            else {
                // TODO: Switch this to a local or alert message
                PHD.alert(gettext("Please enter some search criteria."));
            }
        },

        next: function() {
            if (this.$results) {
                this.$current.removeClass("search-current");
                if (++this.currentIndex >= this.$results.length) {
                    this.currentIndex = 0;
                }
                this.$current = this.$results.eq(this.currentIndex);
                this.expandToMatch(this.$current.addClass("search-current"));
            }
        },

        previous: function() {
            if (this.$results) {
                this.$current.removeClass("search-current");
                if (--this.currentIndex < 0) {
                    this.currentIndex = this.$results.length - 1;
                }
                this.$current = this.$results.eq(this.currentIndex);
                this.expandToMatch(this.$current.addClass("search-current"));
            }
        },

        clear: function() {
            if (this.$results) {
                this.currentIndex = 0;
                this.$results.removeClass("search-found search-current");
                this.$searchMatchLabel.find("span").text('');
                this.setButton(this.$searchFindBut, "show");
                this.setButton(this.$searchClearBut, "hide");
                if(this.$results.length > 1) {
                    this.setButton(this.$searchNextBut, "hide");
                    this.setButton(this.$searchPrevBut, "hide");
                }
                this.$searchMatchLabel.hide();
                this.$searchTextBox.val("");
                this.$results = null;
            }
            this.clearActive();
        },

        clearActive: function() {
            if(this.$active) {
                this.$active.removeClass("tree-node-found");
            }
        },

        goTo: function(id) {
            this.clearActive();
            this.$active = this.$treeview.find("[data-dbid=\"" + id + "\"]").find(".item-name").first().addClass("tree-node-found");
            this.expandToMatch(this.$active);
        },

        expandToMatch: function($match) {
            var $matchParent,
                $parentCtrl,
                $matchContainer;

            // TODO: simplify this
            $matchContainer = $match.closest(".children");
            $matchParent = $matchContainer.prevAll(".header");
            while ($matchParent.length > 0) {
                $parentCtrl = $matchParent.children(".expand-ctrl");
                if (!$parentCtrl.is(".expanded")) {
                    $parentCtrl.addClass("expanded");
                    $matchContainer.addClass("open");
                }
                $matchContainer = $matchParent.closest(".children");
                $matchParent = $matchContainer.prevAll(".header");
            }

            if(this.$scrollContent.height() > this.$scrollBox.height()) {
                this.$scrollBox.scrollTop($match.offset().top - this.$scrollContent.offset().top - 30);
            }
            
            if(this.$scrollContent.width() > this.$scrollBox.width()) {
                this.$scrollBox.scrollLeft($match.offset().left - this.$scrollContent.offset().left);
            }
        }
    };

    PHD.TreeViewSearch = TreeViewSearch;

    var TreeViewHeader = function TreeViewHeader($container, treeview, namespace) {
        if(this instanceof TreeViewHeader) {
            this.$container = $container;
            this.treeview = treeview;
            this.namespace = namespace;
            this.confirmMessage = gettext("You have added items to your inventory; these selections and settings will be lost if you change hypervisor views");
        } else {
            return new TreeViewHeader($container, treeview, namespace).init();
        }
    };

    TreeViewHeader.prototype = {
        init: function() {
            var self = this;

            this.hypervisors = {};

            this.$container.find(".hypervisor-view-select select").each(function(index, Element) {
                var $select = $(Element);

                self.hypervisors[$select[0].name] = $select;
                
                $select.selectmenu({
                    appendTo: ".treeview-header",
                    label: '<img src="' + STATIC_URL + 'images/icon-hv-' + ($select.data("hv")) + '.png">',
                    select: function(event, ui) {
                        event.preventDefault();
                        self.change($(this), ui);
                    },
                    change: function(event, ui) {
                        self.refresh($(this));
                    }
                });

                $select.on("change", function() {
                    self.refresh($(this));
                });
            });

            this.$container.on("click", ".treeview-expand", function() {
                self.treeview.expandAll();
                $(this)
                    .addClass("treeview-collapse")
                    .attr("title", gettext("Collapse All"))
                    .removeClass("treeview-expand");
            });

            this.$container.on("click", ".treeview-collapse", function() {
                self.treeview.collapseAll();
                $(this)
                    .addClass("treeview-expand")
                    .attr("title", gettext("Expand All"))
                    .removeClass("treeview-collapse");
            });

            return this;
        },

        change: function($select, ui) {
            if($select[0].selectedIndex === ui.item.index) {
                return false;
            }

            if(this.treeview.hasSelections()) {
                var promise = PHD.confirmDialog("<p>"+this.confirmMessage+"</p>", { title: gettext("Confirm Hypervisor View Change") });

                promise.done(function() {
                    $select.selectmenu("select", ui.item);
                });
            } else {
                $select.selectmenu("select", ui.item);
            }
        },

        refresh: function($select) {
            var views = _.values(this.hypervisors).map(function(value, index) {
                return value.val();
            });
            $(document).trigger("hypervisorviewchange." + this.namespace, [views]);

            this.$container.find(".treeview-collapse")
                .addClass("treeview-expand")
                .attr("title", gettext("Expand All"))
                .removeClass("treeview-collapse");
        },

        getQueryString: function() {
            return "hypervisor=" + this.$container.find(".hypervisor-view-select select").map(function() {
                return $(this).val();
            }).get().join("&hypervisor=");
        },

        collapseTree: function() {
            this.$container.find(".treeview-collapse")
                .addClass("treeview-expand")
                .attr("title", gettext("Expand All"))
                .removeClass("treeview-collapse");
        }
    };

    PHD.TreeViewHeader = TreeViewHeader;

    var SingleSelectTreeView = function SingleSelectTreeView($treeContainer, selectable) {
        if(this instanceof SingleSelectTreeView) {
            this.$container = $treeContainer; // .treeview
            this.selectable = selectable || [];
            this.selectedID = null;
        } else {
            return new SingleSelectTreeView($treeContainer, selectable).init();
        }
    };

    SingleSelectTreeView.prototype = {

        init: function() {
            var self = this,
                $tree = this.$container;

            this.$universe = this.$container.find(".tree-universe").first();

            // Set up DOM event handlers
            $tree
                .on("click", ".header", function(event) {
                    var $header = $(this),
                        $children = $header.nextAll(".children"),
                        $node = $header.parent();

                    if(self.selectedID != $node.data("dbid") && self.isSelectable($node)) {
                        $tree.find(".selected").removeClass("selected");
                        $header.addClass("selected");
                        self.selectedID = $node.data("dbid");
                        $node.trigger("nodeclick", $node.data("dbid"));
                    } else {
                        // Toggle the treeview item's plus/minus button
                        $header.find(".expand-ctrl").toggleClass("expanded");
                        $children.toggleClass("open")
                    }

                    return false;
                })
                .on("click", ".expand-ctrl", function(event) {
                    var $header = $(this).closest(".header"),
                        $children = $header.nextAll(".children").toggleClass("open");

                    // Toggle the treeview item's plus/minus button
                    $header.find(".expand-ctrl").toggleClass("expanded");
                    $children.trigger("expand");
                    return false;
                });

            return this.disableInvalidChoices();
        },

        disableInvalidChoices: function() {
            
            return this;
        },

        isSelectable: function($node) {
            var type = parseInt($node.data("type"), 10);
            return (this.selectable.indexOf(type) !== -1);
        },

        getSelectionID: function() {
            return this.$container.find(".selected").closest(".checkable").data("dbid");
        },

        expandAll: function() {
            this.$container.find("[data-node=container] > .header .expand-ctrl").addClass("expanded");
            this.$container.find("[data-node=container] > .children").css("display", "block");
            return this;
        },
        getNode: function(id) {
            return this.$container.find(".checkable[data-dbid=\"" + id + "\"]");
        },
        getNodeType: function(id) {
            return this.getNode(id).data("node");
        },
        getNodeName: function(id) {
            return this.getNode(id).children(".header").children(".item-name").html();
        }
    };

    PHD.SingleSelectTreeView = SingleSelectTreeView;

})(jQuery, window.PHD || (window.PHD = {}), window);
